@@ -1,29 +1,30 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2004 Edgewall Software
-# Copyright (C) 2003-2004 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2003-2006 Edgewall Software
+# Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
-# individuals. For exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Jonas Borgstrm <jonas@edgewall.com>
+# Author: Jonas Borgström <jonas@edgewall.com>
 
-from __future__ import generators
 import re
 import time
 
+from trac.config import IntOption
 from trac.core import *
 from trac.perm import IPermissionRequestor
-from trac.util import TracError, escape, format_datetime, Markup
+from trac.util.datefmt import format_datetime
+from trac.util.html import escape, html, Element
 from trac.web import IRequestHandler
 from trac.web.chrome import add_link, add_stylesheet, INavigationContributor
-from trac.wiki import IWikiSyntaxProvider
+from trac.wiki import IWikiSyntaxProvider, wiki_to_link
 
 
 class ISearchSource(Interface):
@@ -35,14 +36,16 @@ class ISearchSource(Interface):
     def get_search_filters(self, req):
         """
         Return a list of filters that this search source supports. Each
-        filter must be a (name, label) tuple, where `name` is the internal
-        name, and `label` is a human-readable name for display.
+        filter must be a (name, label[, default]) tuple, where `name` is the
+        internal name, `label` is a human-readable name for display and
+        `default` is an optional boolean for determining whether this filter
+        is searchable by default.
         """
 
-    def get_search_results(self, req, query, filters):
+    def get_search_results(self, req, terms, filters):
         """
-        Return a list of search results matching `query`. The `filters`
-        parameters is a list of the enabled
+        Return a list of search results matching each search term in `terms`.
+        The `filters` parameters is a list of the enabled
         filters, each item being the name of the tuples returned by
         `get_search_events`.
 
@@ -51,17 +54,35 @@ class ISearchSource(Interface):
         """
 
 
-def query_to_sql(db, q, name):
+def search_terms(q):
+    """
+    Break apart a search query into its various search terms.  Terms are
+    grouped implicitly by word boundary, or explicitly by (single or double)
+    quotes.
+    """
+    results = []
+    for term in re.split('(".*?")|(\'.*?\')|(\s+)', q):
+        if term != None and term.strip() != '':
+            if term[0] == term[-1] == "'" or term[0] == term[-1] == '"':
+                term = term[1:-1]
+            results.append(term)
+    return results
+
+def search_to_sql(db, columns, terms):
     """
     Convert a search query into a SQL condition string and corresponding
     parameters. The result is returned as a (string, params) tuple.
     """
-    if q[0] == q[-1] == "'" or q[0] == q[-1] == '"':
-        keywords = [q[1:-1]]
-    else:
-        keywords = q.split(' ')
-    c = ["%s %s %%s" % (name, db.like())] * len(keywords)
-    return ' AND '.join(c), ['%'+k+'%' for k in keywords]
+    if len(columns) < 1 or len(terms) < 1:
+        raise TracError('Empty search attempt, this should really not happen.')
+
+    likes = ['%s %s' % (i, db.like()) for i in columns]
+    c = ' OR '.join(likes)
+    sql = '(' + ') AND ('.join([c] * len(terms)) + ')'
+    args = []
+    for t in terms:
+        args.extend(['%'+db.like_escape(t)+'%'] * len(columns))
+    return sql, tuple(args)
 
 def shorten_result(text='', keywords=[], maxlen=240, fuzz=60):
     if not text: text = ''
@@ -99,6 +120,9 @@ class SearchModule(Component):
     
     RESULTS_PER_PAGE = 10
 
+    min_query_length = IntOption('search', 'min_query_length', 3,
+        """Minimum length of query string allowed when performing a search.""")
+
     # INavigationContributor methods
 
     def get_active_navigation_item(self, req):
@@ -108,8 +132,7 @@ class SearchModule(Component):
         if not req.perm.has_permission('SEARCH_VIEW'):
             return
         yield ('mainnav', 'search',
-               Markup('<a href="%s" accesskey="4">Search</a>',
-                      self.env.href.search()))
+               html.A('Search', href=req.href.search(), accesskey=4))
 
     # IPermissionRequestor methods
 
@@ -130,7 +153,8 @@ class SearchModule(Component):
             
         filters = [f[0] for f in available_filters if req.args.has_key(f[0])]
         if not filters:
-            filters = [f[0] for f in available_filters]
+            filters = [f[0] for f in available_filters
+                       if len(f) < 3 or len(f) > 2 and f[2]]
                 
         req.hdf['search.filters'] = [
             { 'name': filter[0],
@@ -143,23 +167,22 @@ class SearchModule(Component):
         query = req.args.get('q')
         if query:
             page = int(req.args.get('page', '1'))
-            redir = self.quickjump(query)
-            if redir:
-                req.redirect(redir)
-            elif query.startswith('!'):
+            self.check_quickjump(req, query)
+            if query.startswith('!'):
                 query = query[1:]
+            terms = search_terms(query)
             # Refuse queries that obviously would result in a huge result set
-            if len(query) < 3 and len(query.split()) == 1:
+            if len(terms) == 1 and len(terms[0]) < self.min_query_length:
                 raise TracError('Search query too short. '
-                                'Query must be at least 3 characters long.',
-                                'Search Error')
+                                'Query must be at least %d characters long.' % \
+                                self.min_query_length, 'Search Error')
             results = []
             for source in self.search_sources:
-                results += list(source.get_search_results(req, query, filters))
+                results += list(source.get_search_results(req, terms, filters))
             results.sort(lambda x,y: cmp(y[2], x[2]))
             page_size = self.RESULTS_PER_PAGE
             n = len(results)
-            n_pages = n / page_size + 1
+            n_pages = (n-1) / page_size + 1
             results = results[(page-1) * page_size: page * page_size]
 
             req.hdf['title'] = 'Search Results'
@@ -169,20 +192,18 @@ class SearchModule(Component):
             req.hdf['search.n_pages'] = n_pages
             req.hdf['search.page_size'] = page_size
             if page < n_pages:
-                next_href = self.env.href.search(zip(filters,
-                                                     ['on'] * len(filters)),
-                                                 q=req.args.get('q'),
-                                                 page=page + 1)
+                next_href = req.href.search(zip(filters, ['on'] * len(filters)),
+                                            q=req.args.get('q'), page=page + 1,
+                                            noquickjump=1)
                 add_link(req, 'next', next_href, 'Next Page')
             if page > 1:
-                prev_href = self.env.href.search(zip(filters,
-                                                     ['on'] * len(filters)),
-                                                 q=req.args.get('q'),
-                                                 page=page - 1)
+                prev_href = req.href.search(zip(filters, ['on'] * len(filters)),
+                                            q=req.args.get('q'), page=page - 1,
+                                            noquickjump=1)
                 add_link(req, 'prev', prev_href, 'Previous Page')
-            req.hdf['search.page_href'] = self.env.href.search(zip(filters,
-                                                                   ['on'] * len(filters)),
-                                                               q=req.args.get('q'))
+            req.hdf['search.page_href'] = req.href.search(
+                zip(filters, ['on'] * len(filters)), q=req.args.get('q'),
+                noquickjump=1)
             req.hdf['search.result'] = [
                 { 'href': result[0],
                   'title': result[1],
@@ -194,41 +215,29 @@ class SearchModule(Component):
         add_stylesheet(req, 'common/css/search.css')
         return 'search.cs', None
 
-    def quickjump(self, kwd):
-        if len(kwd.split()) != 1:
-            return None
-        # Ticket quickjump
-        if kwd[0] == '#' and kwd[1:].isdigit():
-            return self.env.href.ticket(kwd[1:])
-        elif kwd[0:len('ticket:')] == 'ticket:' and kwd[len('ticket:'):].isdigit():
-            return self.env.href.ticket(kwd[len('ticket:'):])
-        elif kwd[0:len('bug:')] == 'bug:' and kwd[len('bug:'):].isdigit():
-            return self.env.href.ticket(kwd[len('bug:'):])
-        # Changeset quickjump
-        elif kwd[0] == '[' and kwd[-1] == ']' and kwd[1:-1].isdigit():
-            return self.env.href.changeset(kwd[1:-1])
-        elif kwd[0:len('changeset:')] == 'changeset:' and kwd[len('changeset:'):].isdigit():
-            return self.env.href.changeset(kwd[len('changeset:'):])
-        # Report quickjump
-        elif kwd[0] == '{' and kwd[-1] == '}' and kwd[1:-1].isdigit():
-            return self.env.href.report(kwd[1:-1])
-        elif kwd[0:len('report:')] == 'report:' and kwd[len('report:'):].isdigit():
-            return self.env.href.report(kwd[len('report:'):])
-        # Milestone quickjump
-        elif kwd[0:len('milestone:')] == 'milestone:':
-            return self.env.href.milestone(kwd[len('milestone:'):])
+    def check_quickjump(self, req, kwd):
+        noquickjump = int(req.args.get('noquickjump', '0'))
         # Source quickjump
-        elif kwd[0:len('source:')] == 'source:':
-            return self.env.href.browser(kwd[len('source:'):])
-        # Wiki quickjump
-        elif kwd[0:len('wiki:')] == 'wiki:':
-            r = "((^|(?<=[^A-Za-z]))[!]?[A-Z][a-z/]+(?:[A-Z][a-z/]+)+)"
-            if re.match (r, kwd[len('wiki:'):]):
-                return self.env.href.wiki(kwd[len('wiki:'):])
-        elif len(kwd) > 1 and kwd[0].isupper() and kwd[1].islower():
-            r = "((^|(?<=[^A-Za-z]))[!]?[A-Z][a-z/]+(?:[A-Z][a-z/]+)+)"
-            if re.match (r, kwd):
-                return self.env.href.wiki(kwd)
+        quickjump_href = None
+        if kwd[0] == '/':
+            quickjump_href = req.href.browser(kwd)
+            name = kwd
+            description = 'Browse repository path ' + kwd
+        else:
+            link = wiki_to_link(kwd, self.env, req)
+            if isinstance(link, Element):
+                quickjump_href = link.attr['href']
+                name = link.children
+                description = link.attr.get('title', '')
+        if quickjump_href:
+            if noquickjump:
+                req.hdf['search.quickjump'] = {
+                    'href': quickjump_href,
+                    'name': html.EM(name),
+                    'description': description
+                    }
+            else:
+                req.redirect(quickjump_href)
 
     # IWikiSyntaxProvider methods
     
@@ -238,10 +247,10 @@ class SearchModule(Component):
     def get_link_resolvers(self):
         yield ('search', self._format_link)
 
-    def _format_link(self, formatter, ns, query, label):
-        if query and query[0] == '?':
+    def _format_link(self, formatter, ns, target, label):
+        path, query, fragment = formatter.split_link(target)
+        if query:
             href = formatter.href.search() + query.replace(' ', '+')
         else:
-            href = formatter.href.search(q=query)
-        return '<a class="search" href="%s">%s</a>' % (escape(href), label)
-
+            href = formatter.href.search(q=path)
+        return html.A(label, class_='search', href=href)
