@@ -1,4 +1,4 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2004-2006 Edgewall Software
 # Copyright (C) 2004 Daniel Lundin
@@ -6,11 +6,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Daniel Lundin <daniel@edgewall.com>
 
@@ -33,4 +33,4 @@ class TextileRenderer(Component):
 
     def render(self, req, mimetype, content, filename=None, rev=None):
         import textile
-        return textile.textile(content, encoding='utf-8')
+        return textile.textile(content.encode('utf-8'), encoding='utf-8')
