@@ -1,27 +1,29 @@
 #!/usr/bin/env python
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2005 Edgewall Software
-# Copyright (C) 2003-2005 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2003-2006 Edgewall Software
+# Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
-# individuals. For exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Jonas Borgstrm <jonas@edgewall.com>
+# Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from trac.core import ComponentManager
-from trac.db import SQLiteConnection
-
 import unittest
 
+from trac.config import Configuration
+from trac.core import Component, ComponentManager, ExtensionPoint
+from trac.env import Environment
+from trac.db.sqlite_backend import SQLiteConnection
+
 
 def Mock(bases=(), *initargs, **kw):
     """
@@ -105,27 +107,36 @@ class InMemoryDatabase(SQLiteConnection):
     """
     def __init__(self):
         SQLiteConnection.__init__(self, ':memory:')
-
         cursor = self.cnx.cursor()
 
         from trac.db_default import schema
+        from trac.db.sqlite_backend import _to_sql
         for table in schema:
-            for stmt in SQLiteConnection.to_sql(table):
+            for stmt in _to_sql(table):
                 cursor.execute(stmt)
 
         self.cnx.commit()
 
 
-class EnvironmentStub(ComponentManager):
+class TestConfiguration(Configuration):
+    def __init__(self, filename):
+        Configuration.__init__(self, filename)
+        # insulate us from "real" global trac.ini (ref. #3700)
+        from ConfigParser import ConfigParser
+        self.site_parser = ConfigParser()
+
+
+class EnvironmentStub(Environment):
     """A stub of the trac.env.Environment object for testing."""
 
     def __init__(self, default_data=False, enable=None):
         ComponentManager.__init__(self)
+        Component.__init__(self)
         self.enabled_components = enable
         self.db = InMemoryDatabase()
+        self.path = ''
 
-        from trac.config import Configuration
-        self.config = Configuration(None)
+        self.config = TestConfiguration(None)
 
         from trac.log import logger_factory
         self.log = logger_factory('test')
@@ -135,21 +146,16 @@ class EnvironmentStub(ComponentManager):
         self.abs_href = Href('http://example.org/trac.cgi')
 
         from trac import db_default
-        for section, name, value in db_default.default_config:
-            self.config.set(section, name, value)
         if default_data:
             cursor = self.db.cursor()
-            for table, cols, vals in db_default.data:
+            for table, cols, vals in db_default.get_data(self.db):
                 cursor.executemany("INSERT INTO %s (%s) VALUES (%s)"
                                    % (table, ','.join(cols),
                                       ','.join(['%s' for c in cols])),
                                    vals)
             self.db.commit()
-
-    def component_activated(self, component):
-        component.env = self
-        component.config = self.config
-        component.log = self.log
+            
+        self.known_users = []
 
     def is_component_enabled(self, cls):
         if self.enabled_components is None:
@@ -159,20 +165,49 @@ class EnvironmentStub(ComponentManager):
     def get_db_cnx(self):
         return self.db
 
+    def get_templates_dir(self):
+        return None
+
+    def get_known_users(self, db):
+        return self.known_users
+
+
+def locate(fn):
+    """Locates a binary on the path.
+
+    Returns the fully-qualified path, or None.
+    """
+    import os
+    exec_suffix = os.name == 'nt' and '.exe' or ''
+    
+    for p in ["."] + os.environ['PATH'].split(os.pathsep):
+        f = os.path.join(p, fn + exec_suffix)
+        if os.path.exists(f):
+            return f
+    return None
+
 
 def suite():
     import trac.tests
+    import trac.db.tests
+    import trac.mimeview.tests
     import trac.scripts.tests
     import trac.ticket.tests
+    import trac.util.tests
     import trac.versioncontrol.tests
+    import trac.versioncontrol.web_ui.tests
     import trac.web.tests
     import trac.wiki.tests
 
     suite = unittest.TestSuite()
     suite.addTest(trac.tests.suite())
+    suite.addTest(trac.db.tests.suite())
+    suite.addTest(trac.mimeview.tests.suite())
     suite.addTest(trac.scripts.tests.suite())
     suite.addTest(trac.ticket.tests.suite())
+    suite.addTest(trac.util.tests.suite())
     suite.addTest(trac.versioncontrol.tests.suite())
+    suite.addTest(trac.versioncontrol.web_ui.tests.suite())
     suite.addTest(trac.web.tests.suite())
     suite.addTest(trac.wiki.tests.suite())
 
