@@ -1,4 +1,4 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2005 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
@@ -6,11 +6,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
@@ -266,6 +266,21 @@ class ComponentTestCase(unittest.TestCase):
         self.assertEquals('x', tests.next().test())
         self.assertRaises(StopIteration, tests.next)
 
+    def test_instantiation_doesnt_enable(self):
+        """
+        Make sure that a component disabled by the ComponentManager is not
+        implicitly enabled by instantiating it directly.
+        """
+        from trac.core import ComponentManager
+        class DisablingComponentManager(ComponentManager):
+            def is_component_enabled(self, cls):
+                return False
+        class ComponentA(Component):
+            pass
+        mgr = DisablingComponentManager()
+        instance = ComponentA(mgr)
+        self.assertEqual(None, mgr[ComponentA])
+
 
 def suite():
     return unittest.makeSuite(ComponentTestCase, 'test')
