@@ -1,60 +1,36 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2003-2006 Edgewall Software
-# Copyright (C) 2003-2004 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Jonas Borgstrm <jonas@edgewall.com>
+# Author: Jonas Borgström <jonas@edgewall.com>
 
-from __future__ import generators
 import re
-import types
-import urllib
+from StringIO import StringIO
 
 from trac import util
 from trac.core import *
+from trac.db import get_column_names
 from trac.perm import IPermissionRequestor
+from trac.util import sorted
+from trac.util.datefmt import format_date, format_time, format_datetime, \
+                               http_date
+from trac.util.html import html
+from trac.util.text import unicode_urlencode
 from trac.web import IRequestHandler
 from trac.web.chrome import add_link, add_stylesheet, INavigationContributor
-from trac.wiki import wiki_to_html, IWikiSyntaxProvider
-
-
-class ColumnSorter:
-
-    def __init__(self, columnIndex, asc=1):
-        self.columnIndex = columnIndex
-        self.asc = asc
-
-    def sort(self, x, y):
-        const = -1
-        if not self.asc:
-            const = 1
-
-        # make sure to ignore case in comparisons
-        realX = x[self.columnIndex]
-        if isinstance(realX, (str, unicode)):
-            realX = realX.lower()
-        realY = y[self.columnIndex]
-        if isinstance(realY, (str, unicode)):
-            realY = realY.lower()
-
-        result = 0
-        if realX < realY:
-            result = const * 1
-        elif realX > realY:
-            result = const * -1
-
-        return result
-
+from trac.wiki import wiki_to_html, IWikiSyntaxProvider, Formatter
 
 class ReportModule(Component):
 
@@ -70,8 +46,7 @@ class ReportModule(Component):
         if not req.perm.has_permission('REPORT_VIEW'):
             return
         yield ('mainnav', 'tickets',
-               util.Markup('<a href="%s">View Tickets</a>',
-                           self.env.href.report()))
+               html.A('View Tickets', href=req.href.report()))
 
     # IPermissionRequestor methods  
 
@@ -87,7 +62,7 @@ class ReportModule(Component):
         if match:
             if match.group(1):
                 req.args['id'] = match.group(1)
-            return 1
+            return True
 
     def process_request(self, req):
         req.perm.assert_permission('REPORT_VIEW')
@@ -118,7 +93,7 @@ class ReportModule(Component):
                return resp
 
         if id != -1 or action == 'new':
-            add_link(req, 'up', self.env.href.report(), 'Available Reports')
+            add_link(req, 'up', req.href.report(), 'Available Reports')
 
             # Kludge: Reset session vars created by query module so that the
             # query navigation links on the ticket page don't confuse the user
@@ -131,7 +106,7 @@ class ReportModule(Component):
         from trac.ticket.query import QueryModule
         if req.perm.has_permission('TICKET_VIEW') and \
            self.env.is_component_enabled(QueryModule):
-            req.hdf['report.query_href'] = self.env.href.query()
+            req.hdf['report.query_href'] = req.href.query()
 
         add_stylesheet(req, 'common/css/report.css')
         return 'report.cs', None
@@ -142,28 +117,28 @@ class ReportModule(Component):
         req.perm.assert_permission('REPORT_CREATE')
 
         if req.args.has_key('cancel'):
-            req.redirect(self.env.href.report())
+            req.redirect(req.href.report())
 
         title = req.args.get('title', '')
-        sql = req.args.get('sql', '')
+        query = req.args.get('query', '')
         description = req.args.get('description', '')
         cursor = db.cursor()
-        cursor.execute("INSERT INTO report (title,sql,description) "
-                       "VALUES (%s,%s,%s)", (title, sql, description))
+        cursor.execute("INSERT INTO report (title,query,description) "
+                       "VALUES (%s,%s,%s)", (title, query, description))
         id = db.get_last_id(cursor, 'report')
         db.commit()
-        req.redirect(self.env.href.report(id))
+        req.redirect(req.href.report(id))
 
     def _do_delete(self, req, db, id):
         req.perm.assert_permission('REPORT_DELETE')
 
         if req.args.has_key('cancel'):
-            req.redirect(self.env.href.report(id))
+            req.redirect(req.href.report(id))
 
         cursor = db.cursor()
         cursor.execute("DELETE FROM report WHERE id=%s", (id,))
         db.commit()
-        req.redirect(self.env.href.report())
+        req.redirect(req.href.report())
 
     def _do_save(self, req, db, id):
         """
@@ -173,13 +148,13 @@ class ReportModule(Component):
 
         if not req.args.has_key('cancel'):
             title = req.args.get('title', '')
-            sql = req.args.get('sql', '')
+            query = req.args.get('query', '')
             description = req.args.get('description', '')
             cursor = db.cursor()
-            cursor.execute("UPDATE report SET title=%s,sql=%s,description=%s "
-                           "WHERE id=%s", (title, sql, description, id))
+            cursor.execute("UPDATE report SET title=%s,query=%s,description=%s "
+                           "WHERE id=%s", (title, query, description, id))
             db.commit()
-        req.redirect(self.env.href.report(id))
+        req.redirect(req.href.report(id))
 
     def _render_confirm_delete(self, req, db, id):
         req.perm.assert_permission('REPORT_DELETE')
@@ -188,49 +163,49 @@ class ReportModule(Component):
         cursor.execute("SELECT title FROM report WHERE id = %s", (id,))
         row = cursor.fetchone()
         if not row:
-            raise util.TracError('Report %s does not exist.' % id,
-                                 'Invalid Report Number')
+            raise TracError('Report %s does not exist.' % id,
+                            'Invalid Report Number')
         req.hdf['title'] = 'Delete Report {%s} %s' % (id, row[0])
         req.hdf['report'] = {
             'id': id,
             'mode': 'delete',
             'title': row[0],
-            'href': self.env.href.report(id)
+            'href': req.href.report(id)
         }
 
     def _render_editor(self, req, db, id, copy=False):
         if id == -1:
             req.perm.assert_permission('REPORT_CREATE')
-            title = sql = description = ''
+            title = query = description = ''
         else:
             req.perm.assert_permission('REPORT_MODIFY')
             cursor = db.cursor()
-            cursor.execute("SELECT title,description,sql FROM report "
+            cursor.execute("SELECT title,description,query FROM report "
                            "WHERE id=%s", (id,))
             row = cursor.fetchone()
             if not row:
-                raise util.TracError('Report %s does not exist.' % id,
-                                     'Invalid Report Number')
+                raise TracError('Report %s does not exist.' % id,
+                                'Invalid Report Number')
             title = row[0] or ''
             description = row[1] or ''
-            sql = row[2] or ''
+            query = row[2] or ''
 
         if copy:
             title += ' (copy)'
 
         if copy or id == -1:
             req.hdf['title'] = 'Create New Report'
-            req.hdf['report.href'] = self.env.href.report()
+            req.hdf['report.href'] = req.href.report()
             req.hdf['report.action'] = 'new'
         else:
             req.hdf['title'] = 'Edit Report {%d} %s' % (id, title)
-            req.hdf['report.href'] = self.env.href.report(id)
+            req.hdf['report.href'] = req.href.report(id)
             req.hdf['report.action'] = 'edit'
 
         req.hdf['report.id'] = id
         req.hdf['report.mode'] = 'edit'
         req.hdf['report.title'] = title
-        req.hdf['report.sql'] = sql
+        req.hdf['report.sql'] = query
         req.hdf['report.description'] = description
 
     def _render_view(self, req, db, id):
@@ -243,7 +218,7 @@ class ReportModule(Component):
         for action in [k for k,v in actions.items()
                        if req.perm.has_permission(v)]:
             req.hdf['report.can_' + action] = True
-        req.hdf['report.href'] = self.env.href.report(id)
+        req.hdf['report.href'] = req.href.report(id)
 
         try:
             args = self.get_var_args(req)
@@ -276,9 +251,9 @@ class ReportModule(Component):
         # Convert the header info to HDF-format
         idx = 0
         for col in cols:
-            title=col[0].capitalize()
+            title=col.capitalize()
             prefix = 'report.headers.%d' % idx
-            req.hdf['%s.real' % prefix] = col[0]
+            req.hdf['%s.real' % prefix] = col
             if title.startswith('__') and title.endswith('__'):
                 continue
             elif title[0] == '_' and title[-1] == '_':
@@ -297,7 +272,7 @@ class ReportModule(Component):
             colIndex = None
             hiddenCols = 0
             for x in range(len(cols)):
-                colName = cols[x][0]
+                colName = cols[x]
                 if colName == sortCol:
                     colIndex = x
                 if colName.startswith('__') and colName.endswith('__'):
@@ -306,12 +281,22 @@ class ReportModule(Component):
                 k = 'report.headers.%d.asc' % (colIndex - hiddenCols)
                 asc = req.args.get('asc', None)
                 if asc:
-                    sorter = ColumnSorter(colIndex, int(asc))
-                    req.hdf[k] = asc
+                    asc = int(asc) # string '0' or '1' to int/boolean
                 else:
-                    sorter = ColumnSorter(colIndex)
-                    req.hdf[k] = 1
-                rows.sort(sorter.sort)
+                    asc = 1
+                req.hdf[k] = asc
+                def sortkey(row):
+                    val = row[colIndex]
+                    if isinstance(val, basestring):
+                        val = val.lower()
+                    return val
+                rows = sorted(rows, key=sortkey, reverse=(not asc))
+
+        # Get the email addresses of all known users
+        email_map = {}
+        for username, name, email in self.env.get_known_users():
+            if email:
+                email_map[username] = email
 
         # Convert the rows and cells to HDF-format
         row_idx = 0
@@ -319,8 +304,8 @@ class ReportModule(Component):
             col_idx = 0
             numrows = len(row)
             for cell in row:
-                cell = str(cell)
-                column = cols[col_idx][0]
+                cell = unicode(cell)
+                column = cols[col_idx]
                 value = {}
                 # Special columns begin and end with '__'
                 if column.startswith('__') and column.endswith('__'):
@@ -337,26 +322,33 @@ class ReportModule(Component):
                     value['hidehtml'] = 1
                     column = column[1:]
                 if column in ('ticket', 'id', '_id', '#', 'summary'):
-                    id_cols = [idx for idx, col in util.enum(cols)
-                               if col[0] in ('ticket', 'id', '_id')]
+                    id_cols = [idx for idx, col in enumerate(cols)
+                               if col in ('ticket', 'id', '_id')]
                     if id_cols:
                         id_val = row[id_cols[0]]
-                        value['ticket_href'] = self.env.href.ticket(id_val)
+                        value['ticket_href'] = req.href.ticket(id_val)
                 elif column == 'description':
-                    descr = wiki_to_html(cell, self.env, req, db,
-                                         absurls=(format == 'rss'))
-                    value['parsed'] = format == 'rss' and str(descr) or descr
-                elif column == 'reporter' and cell.find('@') != -1:
-                    value['rss'] = cell
+                    desc = wiki_to_html(cell, self.env, req, db,
+                                        absurls=(format == 'rss'))
+                    value['parsed'] = format == 'rss' and unicode(desc) or desc
+                elif column == 'reporter':
+                    if cell.find('@') != -1:
+                        value['rss'] = cell
+                    elif cell in email_map:
+                        value['rss'] = email_map[cell]
                 elif column == 'report':
-                    value['report_href'] = self.env.href.report(cell)
+                    value['report_href'] = req.href.report(cell)
                 elif column in ('time', 'date','changetime', 'created', 'modified'):
-                    value['date'] = util.format_date(cell)
-                    value['time'] = util.format_time(cell)
-                    value['datetime'] = util.format_datetime(cell)
-                    value['gmt'] = util.http_date(cell)
-                prefix = 'report.items.%d.%s' % (row_idx, str(column))
-                req.hdf[prefix] = str(cell)
+                    if cell == 'None':
+                        value['date'] = value['time'] = cell
+                        value['datetime'] = value['gmt'] = cell
+                    else:
+                        value['date'] = format_date(cell)
+                        value['time'] = format_time(cell)
+                        value['datetime'] = format_datetime(cell)
+                        value['gmt'] = http_date(cell)
+                prefix = 'report.items.%d.%s' % (row_idx, unicode(column))
+                req.hdf[prefix] = unicode(cell)
                 for key in value.keys():
                     req.hdf[prefix + '.' + key] = value[key]
 
@@ -367,10 +359,15 @@ class ReportModule(Component):
         if format == 'rss':
             return 'report_rss.cs', 'application/rss+xml'
         elif format == 'csv':
-            self._render_csv(req, cols, rows)
+            filename = id and 'report_%s.csv' % id or 'report.csv'
+            self._render_csv(req, cols, rows, mimetype='text/csv',
+                             filename=filename)
             return None
         elif format == 'tab':
-            self._render_csv(req, cols, rows, '\t')
+            filename = id and 'report_%s.tsv' % id or 'report.tsv'
+            self._render_csv(req, cols, rows, '\t',
+                             mimetype='text/tab-separated-values',
+                             filename=filename)
             return None
 
         return 'report.cs', None
@@ -383,7 +380,7 @@ class ReportModule(Component):
             params['asc'] = req.args['asc']
         href = ''
         if params:
-            href = '&' + urllib.urlencode(params)
+            href = '&' + unicode_urlencode(params)
         add_link(req, 'alternate', '?format=rss' + href, 'RSS Feed',
                  'application/rss+xml', 'rss')
         add_link(req, 'alternate', '?format=csv' + href,
@@ -395,9 +392,9 @@ class ReportModule(Component):
                      'text/plain')
 
     def execute_report(self, req, db, id, sql, args):
-        sql, args = self.sql_sub_vars(req, sql, args)
+        sql, args = self.sql_sub_vars(req, sql, args, db)
         if not sql:
-            raise util.TracError('Report %s has no SQL query.' % id)
+            raise TracError('Report %s has no SQL query.' % id)
         if sql.find('__group__') == -1:
             req.hdf['report.sorting.enabled'] = 1
 
@@ -408,7 +405,7 @@ class ReportModule(Component):
 
         # FIXME: fetchall should probably not be used.
         info = cursor.fetchall() or []
-        cols = cursor.description or []
+        cols = get_column_names(cursor)
 
         db.rollback()
 
@@ -423,12 +420,12 @@ class ReportModule(Component):
             description = 'This is a list of reports available.'
         else:
             cursor = db.cursor()
-            cursor.execute("SELECT title,sql,description from report "
+            cursor.execute("SELECT title,query,description from report "
                            "WHERE id=%s", (id,))
             row = cursor.fetchone()
             if not row:
-                raise util.TracError('Report %d does not exist.' % id,
-                                     'Invalid Report Number')
+                raise TracError('Report %d does not exist.' % id,
+                                'Invalid Report Number')
             title = row[0] or ''
             sql = row[1]
             description = row[2] or ''
@@ -438,7 +435,7 @@ class ReportModule(Component):
     def get_var_args(self, req):
         report_args = {}
         for arg in req.args.keys():
-            if not arg == arg.upper():
+            if not arg.isupper():
                 continue
             report_args[arg] = req.args.get(arg)
 
@@ -448,36 +445,70 @@ class ReportModule(Component):
 
         return report_args
 
-    def sql_sub_vars(self, req, sql, args):
+    def sql_sub_vars(self, req, sql, args, db=None):
+        if db is None:
+            db = self.env.get_db_cnx()
         values = []
-        def repl(match):
-            aname = match.group(1)
+        def add_value(aname):
             try:
                 arg = args[aname]
             except KeyError:
-                raise util.TracError("Dynamic variable '$%s' not defined." % aname)
+                raise TracError("Dynamic variable '$%s' not defined." \
+                                % aname)
             req.hdf['report.var.' + aname] = arg
             values.append(arg)
+
+        var_re = re.compile("[$]([A-Z]+)")
+
+        # simple parameter substitution outside literal
+        def repl(match):
+            add_value(match.group(1))
             return '%s'
 
-        return re.sub("'?\$([A-Z]+)'?", repl, sql), values
+        # inside a literal break it and concatenate with the parameter
+        def repl_literal(expr):
+            parts = var_re.split(expr[1:-1])
+            if len(parts) == 1:
+                return expr
+            params = parts[1::2]
+            parts = ["'%s'" % p for p in parts]
+            parts[1::2] = ['%s'] * len(params)
+            for param in params:
+                add_value(param)
+            return db.concat(*parts)
 
-    def _render_csv(self, req, cols, rows, sep=','):
+        sql_io = StringIO()
+
+        # break SQL into literals and non-literals to handle replacing
+        # variables within them with query parameters
+        for expr in re.split("('(?:[^']|(?:''))*')", sql):
+            if expr.startswith("'"):
+                sql_io.write(repl_literal(expr))
+            else:
+                sql_io.write(var_re.sub(repl, expr))
+        return sql_io.getvalue(), values
+
+    def _render_csv(self, req, cols, rows, sep=',', mimetype='text/plain',
+                    filename=None):
         req.send_response(200)
-        req.send_header('Content-Type', 'text/plain;charset=utf-8')
+        req.send_header('Content-Type', mimetype + ';charset=utf-8')
+        if filename:
+            req.send_header('Content-Disposition', 'filename=' + filename)
         req.end_headers()
 
-        req.write(sep.join([c[0] for c in cols]) + '\r\n')
+        req.write(sep.join(cols) + '\r\n')
         for row in rows:
-            sanitize = lambda x: str(x).replace(sep,"_") \
-                                       .replace('\n',' ') \
-                                       .replace('\r',' ')
-            req.write(sep.join(map(sanitize, row)) + '\r\n')
+            req.write(sep.join(
+                [unicode(c).replace(sep,"_")
+                 .replace('\n',' ').replace('\r',' ') for c in row]) + '\r\n')
 
     def _render_sql(self, req, id, title, description, sql):
         req.perm.assert_permission('REPORT_SQL_VIEW')
         req.send_response(200)
         req.send_header('Content-Type', 'text/plain;charset=utf-8')
+        if id:
+            req.send_header('Content-Disposition',
+                            'filename=report_%s.sql' % id)
         req.end_headers()
 
         req.write('-- ## %s: %s ## --\n\n' % (id, title))
@@ -491,13 +522,14 @@ class ReportModule(Component):
         yield ('report', self._format_link)
 
     def get_wiki_syntax(self):
-        yield (r"!?\{\d+\}", lambda x, y, z: self._format_link(x, 'report', y[1:-1], y))
-
-    def _format_link(self, formatter, ns, target, label):
-        report, args = target, ''
-        if '?' in target:
-            report, args = target.split('?')
-            args = '?' + args
-        return '<a class="report" href="%s">%s</a>' % (
-               formatter.href.report(report) + args, label)
+        yield (r"!?\{(?P<it_report>%s\s*)\d+\}" % Formatter.INTERTRAC_SCHEME,
+               lambda x, y, z: self._format_link(x, 'report', y[1:-1], y, z))
 
+    def _format_link(self, formatter, ns, target, label, fullmatch=None):
+        intertrac = formatter.shorthand_intertrac_helper(ns, target, label,
+                                                         fullmatch)
+        if intertrac:
+            return intertrac
+        report, args, fragment = formatter.split_link(target)
+        return html.A(label, href=formatter.href.report(report) + args,
+                      class_='report')
