@@ -1,10 +1,29 @@
-from trac.config import Configuration
-from trac.core import TracError
+from trac import core
+from trac.core import TracError, implements
 from trac.ticket.model import Ticket, Component, Milestone, Priority, Type
+from trac.ticket.api import ITicketChangeListener
 from trac.test import EnvironmentStub
 
 import unittest
 
+class TestTicketChangeListener(core.Component):
+    implements(ITicketChangeListener)
+
+    def ticket_created(self, ticket):
+        self.action = 'created'
+        self.ticket = ticket
+
+    def ticket_changed(self, ticket, comment, author, old_values):
+        self.action = 'changed'
+        self.ticket = ticket
+        self.comment = comment
+        self.author = author
+        self.old_values = old_values
+        
+    def ticket_deleted(self, ticket):
+        self.action = 'deleted'
+        self.ticket = ticket
+
 
 class TicketTestCase(unittest.TestCase):
 
@@ -21,18 +40,24 @@ class TicketTestCase(unittest.TestCase):
             ticket[k] = v
         return ticket.insert()
 
-    def test_create_ticket(self):
-        # Multiple test in one method, this sucks
+    def _create_a_ticket(self):
         # 1. Creating ticket
         ticket = Ticket(self.env)
         ticket['reporter'] = 'santa'
         ticket['summary'] = 'Foo'
         ticket['foo'] = 'This is a custom field'
+        return ticket
+
+    def test_create_ticket_1(self):
+        ticket = self._create_a_ticket()
         self.assertEqual('santa', ticket['reporter'])
         self.assertEqual('Foo', ticket['summary'])
         self.assertEqual('This is a custom field', ticket['foo'])
         ticket.insert()
 
+    def test_create_ticket_2(self):
+        ticket = self._create_a_ticket()
+        ticket.insert()
         # Retrieving ticket
         ticket2 = Ticket(self.env, 1)
         self.assertEqual(1, ticket2.id)
@@ -40,11 +65,16 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual('Foo', ticket2['summary'])
         self.assertEqual('This is a custom field', ticket2['foo'])
 
-        # Modifying ticket
+    def _modify_a_ticket(self):
+        ticket2 = self._create_a_ticket()
+        ticket2.insert()
         ticket2['summary'] = 'Bar'
         ticket2['foo'] = 'New value'
         ticket2.save_changes('santa', 'this is my comment')
+        return ticket2
 
+    def test_create_ticket_3(self):
+        self._modify_a_ticket()
         # Retrieving ticket
         ticket3 = Ticket(self.env, 1)
         self.assertEqual(1, ticket3.id)
@@ -52,6 +82,8 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual(ticket3['summary'], 'Bar')
         self.assertEqual(ticket3['foo'], 'New value')
 
+    def test_create_ticket_4(self):
+        ticket3 = self._modify_a_ticket()
         # Testing get_changelog()
         log = ticket3.get_changelog()
         self.assertEqual(len(log), 3)
@@ -60,6 +92,14 @@ class TicketTestCase(unittest.TestCase):
         self.failUnless(log[1][2] in ok_vals)
         self.failUnless(log[2][2] in ok_vals)
 
+    def test_create_ticket_5(self):
+        ticket3 = self._modify_a_ticket()
+        # Testing delete()
+        ticket3.delete()
+        log = ticket3.get_changelog()
+        self.assertEqual(len(log), 0)
+        self.assertRaises(TracError, Ticket, self.env, 1)
+
     def test_ticket_default_values(self):
         """
         Verify that a ticket uses default values specified in the configuration
@@ -175,8 +215,8 @@ class TicketTestCase(unittest.TestCase):
         ticket['component'] = 'bar'
         ticket['milestone'] = 'foo'
         ticket.save_changes('jane', 'Testing', when=42)
-        for t, author, field, old, new in ticket.get_changelog():
-            self.assertEqual((42, 'jane'), (t, author))
+        for t, author, field, old, new, permanent in ticket.get_changelog():
+            self.assertEqual((42, 'jane', True), (t, author, permanent))
             if field == 'component':
                 self.assertEqual(('foo', 'bar'), (old, new))
             elif field == 'milestone':
@@ -193,14 +233,47 @@ class TicketTestCase(unittest.TestCase):
         ticket['component'] = 'bar'
         ticket['component'] = 'foo'
         ticket.save_changes('jane', 'Testing', when=42)
-        for t, author, field, old, new in ticket.get_changelog():
-            self.assertEqual((42, 'jane'), (t, author))
+        for t, author, field, old, new, permanent in ticket.get_changelog():
+            self.assertEqual((42, 'jane', True), (t, author, permanent))
             if field == 'comment':
                 self.assertEqual(('', 'Testing'), (old, new))
             else:
                 self.fail('Unexpected change (%s)'
                           % ((t, author, field, old, new),))
 
+    def test_change_listener_created(self):
+        listener = TestTicketChangeListener(self.env)
+        ticket = self._create_a_ticket()
+        ticket.insert()
+        self.assertEqual('created', listener.action)
+        self.assertEqual(ticket, listener.ticket)
+
+    def test_change_listener_changed(self):
+        listener = TestTicketChangeListener(self.env)
+        data = {'component': 'foo', 'milestone': 'bar'}
+        tkt_id = self._insert_ticket('Hello World', reporter='john', **data)
+
+        ticket = Ticket(self.env, tkt_id)
+        ticket['component'] = 'new component'
+        ticket['milestone'] = 'new milestone'
+
+        comment = 'changing ticket'
+        ticket.save_changes('author', comment)
+
+        self.assertEqual('changed', listener.action)
+        self.assertEqual(comment, listener.comment)
+        self.assertEqual('author', listener.author)
+        for key, value in data.iteritems():
+            self.assertEqual(value, listener.old_values[key])
+
+    def test_change_listener_deleted(self):
+        listener = TestTicketChangeListener(self.env)
+        ticket = self._create_a_ticket()
+        ticket.insert()
+        ticket.delete()
+        self.assertEqual('deleted', listener.action)
+        self.assertEqual(ticket, listener.ticket)
+
 
 class EnumTestCase(unittest.TestCase):
 
