@@ -1,7 +1,7 @@
-from trac.config import Configuration
 from trac.log import logger_factory
-from trac.test import EnvironmentStub
-from trac.ticket.query import Query
+from trac.test import Mock, EnvironmentStub
+from trac.ticket.query import Query, QueryModule
+from trac.wiki.formatter import LinkFormatter
 
 import unittest
 
@@ -21,7 +21,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_ordered_by_id_desc(self):
         query = Query(self.env, order='id', desc=1)
@@ -32,7 +32,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.id,0)=0 DESC,t.id DESC""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_ordered_by_id_verbose(self):
         query = Query(self.env, order='id', verbose=1)
@@ -43,7 +43,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_ordered_by_priority(self):
         query = Query(self.env) # priority is default order
@@ -54,7 +54,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.priority,'')='',priority.value,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_ordered_by_priority_desc(self):
         query = Query(self.env, desc=1) # priority is default order
@@ -65,7 +65,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.priority,'')='' DESC,priority.value DESC,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_ordered_by_version(self):
         query = Query(self.env, order='version')
@@ -77,7 +77,7 @@ FROM ticket AS t
   LEFT OUTER JOIN version ON (version.name=version)
 ORDER BY COALESCE(t.version,'')='',COALESCE(version.time,0)=0,version.time,t.version,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_ordered_by_version_desc(self):
         query = Query(self.env, order='version', desc=1)
@@ -89,7 +89,7 @@ FROM ticket AS t
   LEFT OUTER JOIN version ON (version.name=version)
 ORDER BY COALESCE(t.version,'')='' DESC,COALESCE(version.time,0)=0 DESC,version.time DESC,t.version DESC,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_milestone(self):
         query = Query.from_string(self.env, 'milestone=milestone1', order='id')
@@ -101,7 +101,7 @@ FROM ticket AS t
 WHERE COALESCE(t.milestone,'')=%s
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['milestone1'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_grouped_by_milestone(self):
         query = Query(self.env, order='id', group='milestone')
@@ -113,7 +113,7 @@ FROM ticket AS t
   LEFT OUTER JOIN milestone ON (milestone.name=milestone)
 ORDER BY COALESCE(t.milestone,'')='',COALESCE(milestone.due,0)=0,milestone.due,t.milestone,COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_all_grouped_by_milestone_desc(self):
         query = Query(self.env, order='id', group='milestone', groupdesc=1)
@@ -125,7 +125,7 @@ FROM ticket AS t
   LEFT OUTER JOIN milestone ON (milestone.name=milestone)
 ORDER BY COALESCE(t.milestone,'')='' DESC,COALESCE(milestone.due,0)=0 DESC,milestone.due DESC,t.milestone DESC,COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_grouped_by_priority(self):
         query = Query(self.env, group='priority')
@@ -136,7 +136,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.priority,'')='',priority.value,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_milestone_not(self):
         query = Query.from_string(self.env, 'milestone!=milestone1', order='id')
@@ -148,7 +148,7 @@ FROM ticket AS t
 WHERE COALESCE(t.milestone,'')!=%s
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['milestone1'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_status(self):
         query = Query.from_string(self.env, 'status=new|assigned|reopened',
@@ -161,7 +161,7 @@ FROM ticket AS t
 WHERE COALESCE(t.status,'') IN (%s,%s,%s)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['new', 'assigned', 'reopened'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_owner_containing(self):
         query = Query.from_string(self.env, 'owner~=someone', order='id')
@@ -173,7 +173,7 @@ FROM ticket AS t
 WHERE COALESCE(t.owner,'') LIKE %s
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['%someone%'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_owner_not_containing(self):
         query = Query.from_string(self.env, 'owner!~=someone', order='id')
@@ -185,7 +185,7 @@ FROM ticket AS t
 WHERE COALESCE(t.owner,'') NOT LIKE %s
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['%someone%'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_owner_beginswith(self):
         query = Query.from_string(self.env, 'owner^=someone', order='id')
@@ -197,7 +197,7 @@ FROM ticket AS t
 WHERE COALESCE(t.owner,'') LIKE %s
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['someone%'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_owner_endswith(self):
         query = Query.from_string(self.env, 'owner$=someone', order='id')
@@ -209,7 +209,7 @@ FROM ticket AS t
 WHERE COALESCE(t.owner,'') LIKE %s
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['%someone'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_custom_field(self):
         self.env.config.set('ticket-custom', 'foo', 'text')
@@ -223,7 +223,7 @@ FROM ticket AS t
 WHERE COALESCE(foo.value,'')=%s
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['something'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_grouped_by_custom_field(self):
         self.env.config.set('ticket-custom', 'foo', 'text')
@@ -236,7 +236,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(foo.value,'')='',foo.value,COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_multiple_owners(self):
         query = Query.from_string(self.env, 'owner=someone|someone_else',
@@ -249,7 +249,7 @@ FROM ticket AS t
 WHERE COALESCE(t.owner,'') IN (%s,%s)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['someone', 'someone_else'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_multiple_owners_not(self):
         query = Query.from_string(self.env, 'owner!=someone|someone_else',
@@ -262,7 +262,7 @@ FROM ticket AS t
 WHERE COALESCE(t.owner,'') NOT IN (%s,%s)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(['someone', 'someone_else'], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_multiple_owners_contain(self):
         query = Query.from_string(self.env, 'owner~=someone|someone_else',
@@ -275,7 +275,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 WHERE (COALESCE(t.owner,'') LIKE %s OR COALESCE(t.owner,'') LIKE %s)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_empty_value_contains(self):
         query = Query.from_string(self.env, 'owner~=|', order='id')
@@ -286,7 +286,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_empty_value_startswith(self):
         query = Query.from_string(self.env, 'owner^=|', order='id')
@@ -297,7 +297,7 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
 
     def test_constrained_by_empty_value_endswith(self):
         query = Query.from_string(self.env, 'owner$=|', order='id')
@@ -308,11 +308,29 @@ FROM ticket AS t
   LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=priority)
 ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual([], args)
-        tickets = query.execute()
+        tickets = query.execute(Mock(href=self.env.href))
+
+class QueryLinksTestCase(unittest.TestCase):
+    def setUp(self):
+        self.env = EnvironmentStub(default_data=True)
+        self.query_module = QueryModule(self.env)
+        self.formatter = LinkFormatter(self.env)
+
+    def _format_link(self, query, label):
+        return str(self.query_module._format_link(self.formatter, 'query',
+                                                  query, label))
+
+    def test_empty_query(self):
+        self.assertEqual(self._format_link('', 'label'),
+                         '<em class="error">[Error: Query filter requires '
+                         'field and constraints separated by a "="]</em>')
 
 
 def suite():
-    return unittest.makeSuite(QueryTestCase, 'test')
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(QueryTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(QueryLinksTestCase, 'test'))
+    return suite
 
 if __name__ == '__main__':
     unittest.main()
