@@ -16,7 +16,7 @@
 
 from trac.log import logger_factory
 from trac.test import Mock, InMemoryDatabase
-from trac.versioncontrol import Repository, Changeset, Node
+from trac.versioncontrol import Repository, Changeset, Node, NoSuchChangeset
 from trac.versioncontrol.cache import CachedRepository
 
 import time
@@ -28,22 +28,26 @@ class CacheTestCase(unittest.TestCase):
     def setUp(self):
         self.db = InMemoryDatabase()
         self.log = logger_factory('test')
+        cursor = self.db.cursor()
+        cursor.execute("INSERT INTO system (name, value) VALUES (%s,%s)",
+                       ('youngest_rev', ''))
 
     def test_initial_sync_with_empty_repos(self):
-        changeset = Mock(Changeset, 0, '', '', 42000,
-                         get_changes=lambda: [])
+        def no_changeset(rev):
+            raise NoSuchChangeset(rev)
+
         repos = Mock(Repository, 'test-repos', None, self.log,
-                     get_changeset=lambda x: changeset,
-                     get_oldest_rev=lambda: 0,
+                     get_changeset=no_changeset,
+                     get_oldest_rev=lambda: 1,
                      get_youngest_rev=lambda: 0,
-                     normalize_rev=lambda x: x,
+                     normalize_rev=no_changeset,
                      next_rev=lambda x: None)
         cache = CachedRepository(self.db, repos, None, self.log)
         cache.sync()
 
         cursor = self.db.cursor()
         cursor.execute("SELECT rev,time,author,message FROM revision")
-        self.assertEquals(('0', 42000, '', ''), cursor.fetchone())
+        self.assertEquals(None, cursor.fetchone())
         cursor.execute("SELECT COUNT(*) FROM node_change")
         self.assertEquals(0, cursor.fetchone()[0])
 
@@ -87,6 +91,7 @@ class CacheTestCase(unittest.TestCase):
                            "VALUES ('1',%s,%s,%s,%s,%s)",
                            [('trunk', 'D', 'A', None, None),
                             ('trunk/README', 'F', 'A', None, None)])
+        cursor.execute("UPDATE system SET value='1' WHERE name='youngest_rev'")
 
         changes = [('trunk/README', Node.FILE, Changeset.EDIT, 'trunk/README', 1)]
         changeset = Mock(Changeset, 2, 'Update', 'joe', 42042,
@@ -94,7 +99,9 @@ class CacheTestCase(unittest.TestCase):
         repos = Mock(Repository, 'test-repos', None, self.log,
                      get_changeset=lambda x: changeset,
                      get_youngest_rev=lambda: 2,
-                     next_rev=lambda x: int(x) == 1 and 2 or None)
+                     get_oldest_rev=lambda: 0,
+                     normalize_rev=lambda x: x,                    
+                     next_rev=lambda x: x and int(x) == 1 and 2 or None)
         cache = CachedRepository(self.db, repos, None, self.log)
         cache.sync()
 
@@ -119,11 +126,14 @@ class CacheTestCase(unittest.TestCase):
                            "VALUES ('1',%s,%s,%s,%s,%s)",
                            [('trunk', 'D', 'A', None, None),
                             ('trunk/README', 'F', 'A', None, None)])
+        cursor.execute("UPDATE system SET value='1' WHERE name='youngest_rev'")
 
         repos = Mock(Repository, 'test-repos', None, self.log,
                      get_changeset=lambda x: None,
                      get_youngest_rev=lambda: 1,
-                     next_rev=lambda x: None, normalize_rev=lambda rev: rev)
+                     get_oldest_rev=lambda: 0,
+                     next_rev=lambda x: None,
+                     normalize_rev=lambda rev: rev)
         cache = CachedRepository(self.db, repos, None, self.log)
         self.assertEqual('1', cache.youngest_rev)
         changeset = cache.get_changeset(1)
