@@ -1,7 +1,7 @@
 import unittest
 
-from trac.web.tests import api, auth, cgi_frontend, chrome, clearsilver, href, \
-                           session
+from trac.web.tests import api, auth, cgi_frontend, chrome, clearsilver, \
+                           href, session, wikisyntax
 
 def suite():
 
@@ -13,6 +13,7 @@ def suite():
     suite.addTest(clearsilver.suite())
     suite.addTest(href.suite())
     suite.addTest(session.suite())
+    suite.addTest(wikisyntax.suite())
     return suite
 
 if __name__ == '__main__':
