@@ -1,75 +1,81 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005 Edgewall Software
-# Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
+# Copyright (C) 2005-2006 Edgewall Software
+# Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
 import imp
 import inspect
 import os
 import re
-
 try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+    set
+except NameError:
+    from sets import Set as set
+from StringIO import StringIO
 
 from trac.config import default_dir
 from trac.core import *
-from trac.util import escape, format_date, Markup
+from trac.util import sorted
+from trac.util.datefmt import format_date
+from trac.util.html import escape, html, Markup
 from trac.wiki.api import IWikiMacroProvider, WikiSystem
 from trac.wiki.model import WikiPage
+from trac.web.chrome import add_stylesheet
 
 
-class TitleIndexMacro(Component):
-    """
-    Inserts an alphabetic list of all wiki pages into the output.
+class WikiMacroBase(Component):
+    """Abstract base class for wiki macros."""
+
+    implements(IWikiMacroProvider)
+    abstract = True
+
+    def get_macros(self):
+        """Yield the name of the macro based on the class name."""
+        name = self.__class__.__name__
+        if name.endswith('Macro'):
+            name = name[:-5]
+        yield name
+
+    def get_macro_description(self, name):
+        """Return the subclass's docstring."""
+        return inspect.getdoc(self.__class__)
+
+    def render_macro(self, req, name, content):
+        raise NotImplementedError
+
+
+class TitleIndexMacro(WikiMacroBase):
+    """Inserts an alphabetic list of all wiki pages into the output.
 
     Accepts a prefix string as parameter: if provided, only pages with names
     that start with the prefix are included in the resulting list. If this
     parameter is omitted, all pages are listed.
     """
-    implements(IWikiMacroProvider)
-
-    def get_macros(self):
-        yield 'TitleIndex'
-
-    def get_macro_description(self, name):
-        return inspect.getdoc(TitleIndexMacro)
 
     def render_macro(self, req, name, content):
         prefix = content or None
 
         wiki = WikiSystem(self.env)
-        pages = list(wiki.get_pages(prefix))
-        pages.sort()
 
-        buf = StringIO()
-        buf.write('<ul>')
-        for page in pages:
-            buf.write('<li><a href="%s">' % escape(self.env.href.wiki(page)))
-            buf.write(escape(page))
-            buf.write('</a></li>\n')
-        buf.write('</ul>')
-
-        return buf.getvalue()
+        return html.UL([html.LI(html.A(wiki.format_page_name(page),
+                                       href=req.href.wiki(page)))
+                        for page in sorted(wiki.get_pages(prefix))])
 
 
-class RecentChangesMacro(Component):
-    """
-    Lists all pages that have recently been modified, grouping them by the day
-    they were last modified.
+class RecentChangesMacro(WikiMacroBase):
+    """Lists all pages that have recently been modified, grouping them by the
+    day they were last modified.
 
     This macro accepts two parameters. The first is a prefix string: if
     provided, only pages with names that start with the prefix are included in
@@ -79,13 +85,6 @@ class RecentChangesMacro(Component):
     For example, specifying a limit of 5 will result in only the five most
     recently changed pages to be included in the list.
     """
-    implements(IWikiMacroProvider)
-
-    def get_macros(self):
-        yield 'RecentChanges'
-
-    def get_macro_description(self, name):
-        return inspect.getdoc(RecentChangesMacro)
 
     def render_macro(self, req, name, content):
         prefix = limit = None
@@ -99,38 +98,46 @@ class RecentChangesMacro(Component):
         db = self.env.get_db_cnx()
         cursor = db.cursor()
 
-        sql = 'SELECT name, max(time) FROM wiki'
+        sql = 'SELECT name, ' \
+              '  max(version) AS max_version, ' \
+              '  max(time) AS max_time ' \
+              'FROM wiki'
         args = []
         if prefix:
             sql += ' WHERE name LIKE %s'
             args.append(prefix + '%')
-        sql += ' GROUP BY name ORDER BY max(time) DESC'
+        sql += ' GROUP BY name ORDER BY max_time DESC'
         if limit:
             sql += ' LIMIT %s'
             args.append(limit)
         cursor.execute(sql, args)
 
-        buf = StringIO()
+        entries_per_date = []
         prevdate = None
-
-        for name, time in cursor:
+        for name, version, time in cursor:
             date = format_date(time)
             if date != prevdate:
-                if prevdate:
-                    buf.write('</ul>')
-                buf.write('<h3>%s</h3><ul>' % date)
                 prevdate = date
-            buf.write('<li><a href="%s">%s</a></li>\n'
-                      % (escape(self.env.href.wiki(name)), escape(name)))
-        if prevdate:
-            buf.write('</ul>')
+                entries_per_date.append((date, []))
+            entries_per_date[-1][1].append((name, int(version)))
 
-        return buf.getvalue()
+        wiki = WikiSystem(self.env)
+        return html.DIV(
+            [html.H3(date) +
+             html.UL([html.LI(
+            html.A(wiki.format_page_name(name), href=req.href.wiki(name)),
+            ' ',
+            version > 1 and 
+            html.SMALL('(', html.A('diff',
+                                   href=req.href.wiki(name, action='diff',
+                                                      version=version)), ')') \
+            or None)
+                      for name, version in entries])
+             for date, entries in entries_per_date])
 
 
-class PageOutlineMacro(Component):
-    """
-    Displays a structural outline of the current wiki page, each item in the
+class PageOutlineMacro(WikiMacroBase):
+    """Displays a structural outline of the current wiki page, each item in the
     outline being a link to the corresponding heading.
 
     This macro accepts three optional parameters:
@@ -149,13 +156,6 @@ class PageOutlineMacro(Component):
        causes the outline to be rendered in a box that is by default floated to
        the right side of the other content.
     """
-    implements(IWikiMacroProvider)
-
-    def get_macros(self):
-        yield 'PageOutline'
-
-    def get_macro_description(self, name):
-        return inspect.getdoc(PageOutlineMacro)
 
     def render_macro(self, req, name, content):
         from trac.wiki.formatter import wiki_to_outline
@@ -192,9 +192,8 @@ class PageOutlineMacro(Component):
         return buf.getvalue()
 
 
-class ImageMacro(Component):
-    """
-    Embed an image in wiki-formatted text.
+class ImageMacro(WikiMacroBase):
+    """Embed an image in wiki-formatted text.
     
     The first argument is the file specification. The file specification may
     reference attachments or files in three ways:
@@ -207,7 +206,7 @@ class ImageMacro(Component):
        within that wiki page or a ticket.
     
     Also, the file specification may refer to repository files, using the
-    `source:file` syntax.
+    `source:file` syntax (`source:file@rev` works also).
     
     The remaining arguments are optional and allow configuring the attributes
     and style of the rendered `<img>` element:
@@ -244,13 +243,6 @@ class ImageMacro(Component):
     ''Adapted from the Image.py macro created by Shun-ichi Goto
     <gotoh@taiyo.co.jp>''
     """
-    implements(IWikiMacroProvider)
-
-    def get_macros(self):
-        yield 'Image'
-
-    def get_macro_description(self, name):
-        return inspect.getdoc(ImageMacro)
 
     def render_macro(self, req, name, content):
         # args will be null if the macro is called without parenthesis.
@@ -263,7 +255,8 @@ class ImageMacro(Component):
             raise Exception("No argument.")
         filespec = args[0]
         size_re = re.compile('[0-9]+%?$')
-        attr_re = re.compile('(align|border|width|height|alt|title|longdesc|class|id|usemap)=(.+)')
+        attr_re = re.compile('(align|border|width|height|alt'
+                             '|title|longdesc|class|id|usemap)=(.+)')
         quoted_re = re.compile("(?:[\"'])(.*)(?:[\"'])$")
         attr = {}
         style = {}
@@ -277,6 +270,9 @@ class ImageMacro(Component):
             if arg == 'nolink':
                 nolink = True
                 continue
+            if arg in ('left', 'right', 'top', 'bottom'):
+                style['float'] = arg
+                continue
             match = attr_re.match(arg)
             if match:
                 key, val = match.groups()
@@ -288,7 +284,7 @@ class ImageMacro(Component):
                 elif key == 'border':
                     style['border'] = ' %dpx solid' % int(val);
                 else:
-                    attr[key] = val
+                    attr[str(key)] = val # will be used as a __call__ keyword
 
         # parse filespec argument to get module and id if contained.
         parts = filespec.split(':')
@@ -307,8 +303,11 @@ class ImageMacro(Component):
                 browser_links = []
             if parts[0] in browser_links:   # source:path
                 module, file = parts
-                url = self.env.href.browser(file)
-                raw_url = self.env.href.browser(file, format='raw')
+                rev = None
+                if '@' in file:
+                    file, rev = file.split('@')
+                url = req.href.browser(file, rev=rev)
+                raw_url = req.href.browser(file, rev=rev, format='raw')
                 desc = filespec
             else: # #ticket:attachment or WikiPage:attachment
                 # FIXME: do something generic about shorthand forms...
@@ -317,8 +316,10 @@ class ImageMacro(Component):
                     module = 'ticket'
                     id = id[1:]
                 elif id == 'htdocs':
-                    raw_url = url = self.env.href.chrome('site', file)
+                    raw_url = url = req.href.chrome('site', file)
                     desc = os.path.basename(file)
+                elif id in ('http', 'https', 'ftp'): # external URLs
+                    raw_url = url = desc = id+':'+file
                 else:
                     module = 'wiki'
         elif len(parts) == 1:               # attachment
@@ -339,27 +340,22 @@ class ImageMacro(Component):
         if not url: # this is an attachment
             from trac.attachment import Attachment
             attachment = Attachment(self.env, module, id, file)
-            url = attachment.href()
-            raw_url = attachment.href(format='raw')
+            url = attachment.href(req)
+            raw_url = attachment.href(req, format='raw')
             desc = attachment.description
         for key in ['title', 'alt']:
             if desc and not attr.has_key(key):
                 attr[key] = desc
-        a_style = 'padding:0; border:none' # style of anchor
-        img_attr = ' '.join(['%s="%s"' % (k, escape(v))
-                             for k, v in attr.iteritems()])
         if style:
-            img_style = '; '.join(['%s:%s' % (k, escape(v))
-                                   for k, v in style.iteritems()])
-            img_attr += ' style="%s"' % img_style
-        result = Markup('<img src="%%s" %s />' % img_attr, raw_url).sanitize()
+            attr['style'] = '; '.join(['%s:%s' % (k, escape(v))
+                                       for k, v in style.iteritems()])
+        result = Markup(html.IMG(src=raw_url, **attr)).sanitize()
         if not nolink:
-            result = Markup('<a href="%s" style="%s">%s</a>',
-                            url, a_style, result)
+            result = html.A(result, href=url, style='padding:0; border:none')
         return result
 
 
-class MacroListMacro(Component):
+class MacroListMacro(WikiMacroBase):
     """Displays a list of all installed Wiki macros, including documentation if
     available.
     
@@ -369,33 +365,57 @@ class MacroListMacro(Component):
     Note that this macro will not be able to display the documentation of
     macros if the `PythonOptimize` option is enabled for mod_python!
     """
-    implements(IWikiMacroProvider)
-
-    def get_macros(self):
-        yield 'MacroList'
-
-    def get_macro_description(self, name):
-        return inspect.getdoc(MacroListMacro)
 
     def render_macro(self, req, name, content):
-        from trac.wiki.formatter import wiki_to_html
-        from trac.wiki import WikiSystem
-        buf = StringIO()
-        buf.write("<dl>")
-
+        from trac.wiki.formatter import wiki_to_html, system_message
         wiki = WikiSystem(self.env)
-        for macro_provider in wiki.macro_providers:
-            for macro_name in macro_provider.get_macros():
-                if content and macro_name != content:
-                    continue
-                buf.write("<dt><code>[[%s]]</code></dt>" % escape(macro_name))
-                description = macro_provider.get_macro_description(macro_name)
-                if description:
-                    buf.write("<dd>%s</dd>" % wiki_to_html(description,
-                                                           self.env, req))
 
-        buf.write("</dl>")
-        return buf.getvalue()
+        def get_macro_descr():
+            for macro_provider in wiki.macro_providers:
+                for macro_name in macro_provider.get_macros():
+                    if content and macro_name != content:
+                        continue
+                    try:
+                        descr = macro_provider.get_macro_description(macro_name)
+                        descr = wiki_to_html(descr or '', self.env, req)
+                    except Exception, e:
+                        descr = Markup(system_message(
+                            "Error: Can't get description for macro %s" \
+                            % macro_name, e))
+                    yield (macro_name, descr)
+
+        return html.DL([(html.DT(html.CODE('[[',macro_name,']]'),
+                                 id='%s-macro' % macro_name),
+                         html.DD(description))
+                        for macro_name, description in get_macro_descr()])
+
+
+class TracIniMacro(WikiMacroBase):
+    """Produce documentation for Trac configuration file.
+
+    Typically, this will be used in the TracIni page.
+    Optional arguments are a configuration section filter,
+    and a configuration option name filter: only the configuration
+    options whose section and name start with the filters are output.
+    """
+
+    def render_macro(self, req, name, filter):
+        from trac.config import Option
+        from trac.wiki.formatter import wiki_to_html, wiki_to_oneliner
+        filter = filter or ''
+
+        sections = set([section for section, option in Option.registry.keys()
+                        if section.startswith(filter)])
+
+        return html.DIV(class_='tracini')(
+            [(html.H2('[%s]' % section, id='%s-section' % section),
+              html.TABLE(class_='wiki')(
+                  html.TBODY([html.TR(html.TD(html.TT(option.name)),
+                                      html.TD(wiki_to_oneliner(option.__doc__,
+                                                               self.env)))
+                              for option in Option.registry.values()
+                              if option.section == section])))
+             for section in sorted(sections)])
 
 
 class UserMacroProvider(Component):
